// --- Seascape ---------------------------------------------------------------
//
// two player network game with X11 Interface
// Copyright (C) 04.09.97 Marco Dettori and Oliver Obst
// $Id: main.C,v 0.2 1998/03/16 10:22:25 oliver Exp $

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// To contact the authors, write to one of the following email addresses:
// marco <lespaul@uni-koblenz.de>
// oliver <frvit@uni-koblenz.de>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <strings.h>

#ifdef HAVE_KDE
#include <kapp.h>
#else
#include <qapp.h>
#endif

#include <qpushbt.h>
#include <qfont.h>
#include <qmsgbox.h>

#include "main.h"
#include "constants.h"
#include "seascape.xpm"
#include "version.h"

ShipKill::ShipKill( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
        setMinimumSize( 340, 200 );

        bye = new QPushButton( "Quit", this, "quit" );
        bye->setGeometry( 10, btny, btnwidth, btnheight );
        bye->setFont( QFont( btnfont, btnfsize, QFont::Bold ) );

        reset = new QPushButton( "Reset", this, "reset" );
        reset->setGeometry( bye->x() + btnwidth, btny, btnwidth, btnheight );
        reset->setFont( QFont( btnfont, btnfsize, QFont::Bold ) );
        resetOk = TRUE;

        ok = new QPushButton( "OK", this, "ok" );
        ok->setGeometry( reset->x() + btnwidth, btny, btnwidth, btnheight );
        ok->setFont( QFont( btnfont, btnfsize, QFont::Light ) );
        okOk = FALSE;

        t1 = new MyOcean( edges, ships, fleet, statuslen, this );
        t1->setLineWidth( 1 );
        t1->setFrameStyle( QFrame::Box | QFrame::Raised );
        t1->setBackgroundColor( lightGray );        

        
        t2 = new OpOcean( edges, ships, fleet, statuslen, this );
        t2->setLineWidth( 1 );
        t2->setFrameStyle( QFrame::Box | QFrame::Raised );
        t2->setBackgroundColor( lightGray );

        label = new QPushButton( this );
        label->setText( "Set your ships!" );
        label->setGeometry( 10, btny + btnheight, 3 * btnwidth, 21 );

        pixmap = new QPixmap((const char **) seascape);
        icon = new QPushButton( this );
        icon->setAutoResize( TRUE );
        icon->setPixmap( *pixmap );
        
        proto = new SeaProto();

        
        connect( bye, SIGNAL( clicked() ), SLOT( surrender() ) );
        connect( bye, SIGNAL( clicked() ), qApp, SLOT( quit() ) );

        connect( reset, SIGNAL( clicked() ), SLOT( resetS() ) );

        connect( ok, SIGNAL( clicked() ), SLOT( okS() ) );
        connect( ok, SIGNAL( clicked() ), t1, SLOT( clearWater() ) );

        connect( proto, SIGNAL( reporter( int, int ) ),
                 t2, SLOT( report( int, int ) ) );
        connect( proto, SIGNAL( request( int ) ),
                 t1, SLOT( peek( int ) ) );
        connect( proto, SIGNAL( reporter( int, int ) ),
                 this, SLOT( opToggle( int, int ) ) );
        connect( proto, SIGNAL( defeat() ), SLOT( survive() ) );

        connect( t1, SIGNAL( ready() ), SLOT( okOrResetS() ) );
        connect( t1, SIGNAL( reporter( int, int ) ),
                 proto, SLOT( writeReport( int, int ) ) );
        connect( t1, SIGNAL( reporter( int, int ) ),
                 this, SLOT( myToggle( int, int ) ) );

        connect( t2, SIGNAL( request( int ) ),
                 proto, SLOT( writePeek( int ) ) );

        connect( label, SIGNAL( pressed() ), SLOT( showInfo() ) );
        connect( label, SIGNAL( released() ), SLOT( restoreInfo() ) );

        connect( icon, SIGNAL( clicked() ), SLOT( about() ) );        
}

// --- public -----------------------------------------------------------------
// ShipKill :: setParameter

void ShipKill :: setParameter( int option, const char *parameter )
{
        switch( option ) {
        case 0 :
                proto->setServer( parameter );
                break;
        case 1 :
                proto->setPort( atoi( parameter ) );
                break;
        }
}


// --- protected --------------------------------------------------------------
// ShipKill :: resizeEvent

void ShipKill::resizeEvent( QResizeEvent * )
{
        icon->setGeometry( width() - pixmap->width() - 10 , 5,
                           pixmap->width(), pixmap->height() );
        
        short fleetwidth        = 48 * width() / 100 - 10;
        short fleetheight       = fleetwidth * 5 / 4;
        short fleetypos         = 10 + icon->y() + icon->height();

        if ( fleetypos + fleetheight + 10 > height() ) {
                fleetheight = height() - fleetypos - 10;
                fleetwidth  = fleetheight * 2 / 3;
        }

        if ( fleetwidth < 160 ) {
                fleetwidth = 160;
                fleetheight = 160 * 3 / 2;
        }        
        
        t1->setGeometry( 10, fleetypos, fleetwidth, fleetheight );

        t2->setGeometry( width() - fleetwidth - 10, fleetypos,
                         fleetwidth, fleetheight );

}

// --- protected slot ---------------------------------------------------------
// ShipKill :: okOrResetS

void ShipKill :: okOrResetS()
{
        ok->setFont( QFont( btnfont, btnfsize, QFont::Bold ) );
        label->setText( "Press OK and await 2nd player." );
        okOk = TRUE;
}

// --- protected slot ---------------------------------------------------------
// ShipKill :: resetS()

void ShipKill :: resetS()
{
        if ( resetOk ) {
                ok->setFont( QFont( btnfont, btnfsize, QFont::Light ) );
                okOk = FALSE;

                t1->reset();
                t2->reset();
                label->setText( "Set your ships!" );
        }
}

// --- protected slot ---------------------------------------------------------
// ShipKill :: ok()
//
// prepare to start the show.

void ShipKill :: okS()
{
        if ( okOk ) {
                label->setText( "Please wait for a 2nd player." );
                ok->setFont( QFont( btnfont, btnfsize, QFont::Light ) );

                okOk = FALSE;
                resetOk = FALSE;
                
                uname( &uts );
                strcpy( myname, cuserid( 0 ) );
                strcat( myname, "@" );
                strcat( myname, uts.nodename );
                
                switch ( proto->seaOpen( opname, myname ) ) {
                case 0 :
                        t2->setState( OpOcean::OPTURN );
                        label->setText( "Your opponent begins." );
                        break;
                case 1 :
                        t2->setState( OpOcean::MYTURN );
                        label->setText( "Make the first move!" );
                        break;
                default :
                        t2->setState( OpOcean::WAITING );
                        reset->setFont( QFont( btnfont,
                                               btnfsize,
                                               QFont::Bold ) );
                        resetOk = TRUE;
                }
        }
}


// --- protected slot ---------------------------------------------------------
// ShipKill :: myToggle

void ShipKill :: myToggle( int, int result )
{
        switch ( result ) {
        case MISSED : 
                t2->setState( OpOcean::MYTURN );
                label->setText( "OK, it's your turn." );
                break;
        case DEFEAT :
                t2->setState( OpOcean::WAITING );
                label->setText( "Sorry, you lost!" );
                proto->seaClose();
                reset->setFont( QFont( btnfont, btnfsize, QFont::Bold ) );
                resetOk = TRUE;
                break;
        default :
                t2->setState( OpOcean::OPTURN );
                label->setText( "It's your opponents turn." );
        }
}

// --- protected slot ---------------------------------------------------------
// ShipKill :: opToggle

void ShipKill :: opToggle( int, int result )
{
        switch ( result ) {
        case MISSED :
                label->setText( "It's your opponents turn." );
                break;
        case HIT :
                label->setText( "Hit! So it's your turn again.");
                break;
        case SINK :
                label->setText( "YES! Get the next one!" );
                break;
        case DEFEAT :
                label->setText( "Hey! You won!" );
                proto->seaClose();
                reset->setFont( QFont( btnfont, btnfsize, QFont::Bold ) );
                resetOk = TRUE;
        }
}

// --- protected slot ---------------------------------------------------------
//
// ShipKill :: quit

void ShipKill :: surrender()
{
        if ( proto->connected() )
                proto->seaClose( TRUE );        
}


// --- protected slot ---------------------------------------------------------
//
// ShipKill :: survive()
//
// the opponent gave up

void ShipKill :: survive()
{
        label->setText( "Your opponent left the game!" );
        proto->seaClose();
        reset->setFont( QFont( btnfont, btnfsize, QFont::Bold ) );
        resetOk = TRUE;
}

// --- protected slot ---------------------------------------------------------
//
// ShipKill :: showInfo()
//
// display opponents name in the status bar

void ShipKill :: showInfo()
{
        strcpy( saver, label->text() );
        if ( proto->connected() )
                label->setText( opname );
        else
                label->setText( "You are not connected yet!" );
}

// --- protected slot ---------------------------------------------------------
//
// ShipKill :: restoreInfo()
//
// restore former text of the status bar

void ShipKill :: restoreInfo()
{
        label->setText( saver );
}


// --- protected slot ---------------------------------------------------------
//
// ShipKill :: about()
//
// open a widget and display copyright and other useful information

void ShipKill :: about()
{
        char info[] =
                "\n\n"
                "The latest version and further information is available on\n"
                "http://www.uni-koblenz.de/~frvit/SEASCAPE/index.html\n"
                "Please send bug reports or requests to <frvit@uni-koblenz.de>.\n\n"
                "This program is free software; you can redistribute it and/or\n"
                "modify it under the terms of the GNU General Public License\n"
                "as published by the Free Software Foundation; either version 2\n"
                "of the License, or (at your option) any later version.\n\n"
                "This program is distributed in the hope that it will be useful,\n"
                "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
                "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
                "GNU General Public License for more details.\n\n"
                "You should have received a copy of the GNU General Public License\n"
                "along with this program; if not, write to the Free Software\n"
                "Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.";

        char message[ strlen(info)
                    + strlen(Versiontxt)
                    + strlen(Copyright) + 3 ];

        strcpy((char *)message, Versiontxt);
        strcat((char *)message, "\n");
        strcat((char *)message, Copyright);
        strcat((char *)message, info);
        
        QMessageBox infobox;
        infobox.setFont(QFont( "times", 10));
        infobox.about(this, "About Seascape", message);
}

                                

int main( int argc, char **argv )
{
        cout << Versiontxt << "\n"
             << Copyright << "\n";

#ifdef HAVE_KDE
        KApplication a(argc, argv);
#else        
        QApplication a(argc, argv);
#endif        

        ShipKill sk;
        sk.setGeometry( 100, 100, 480, 430 );
        sk.setBackgroundColor( lightGray );
        a.setMainWidget( &sk );

        int i=1;        
        while ( i < a.argc() ) {
                for ( int j=0; j < parac; j++ ) 
                        if ( strcmp( a.argv()[ i ], parav[ j ] ) == 0 ) 
                                if ( i < a.argc() - 1 )
                                        sk.setParameter( j, a.argv()[ ++i ] );
                                else
                                        sk.setParameter( j, "" );
                
                i++;
        }
        
        sk.show();
        return a.exec();
}
